

Tablón: Server for the Parallel Computing Course
================================================================================

* Javier Fresno 2013-2016
* Last version: 5.5
* Arturo Gonzalez-Escribano 2017
* Last version: 5.6


Dependencies
---------------------------------------------------------------------

* Pyhton2: It works with python2.7.

* Python PIP: The command to install the python modules.
	- Ubuntu: apt-get install python-dev   python-pip
	- Centos: yum     install python-devel python-pip
	- Manually:
		wget --no-check-certificate https://bootstrap.pypa.io/get-pip.py
		python get-pip.py
	
* Jinja2: This is the HTML template system.
	- Using PIP:
		pip install jinja2
	- If you don't like root, you can install the modules only for your user.
		pip install --user jinja2

* SRP: Secure Remote Password protocol ( https://github.com/cocagne/pysrp)
	SRP is a cryptographically strong authentication protocol for password-based,
	mutual authentication over an insecure network connection.
	- I have included the python version with the Tablón code but if you install
	  this library, Tablón will use the system library that is faster because
	  is implemented using C.
	- Install:
		pip install srp


* Pygal: Is a python module to print plots.
	- If it is not installed the stats page will not work.
	- Install:
		pip install pygal

* MySQL: This is the used database system
	- Ubuntu: apt-get install mysql-server
	- Centos: https://www.linode.com/docs/databases/mysql/how-to-install-mysql-on-centos-7
		sudo yum install mysql-server
		sudo yum install mysql-devel
		sudo systemctl start mysqld

	- Install python interface:
		pip install MySQL-python

	- Create the database (e.g. tablon1617)
		sudo mysql -u root
		show databases;
		CREATE DATABASE tablon1617;
		CREATE USER 'tablon'@'localhost' IDENTIFIED BY '......';
		GRANT ALL PRIVILEGES ON tablon.* TO 'tablon'@'localhost';

	- @arturo2017: Build the tables
		mysql -u tablon1617 -p..... < ScriptsSQL/db_createStructure.sql

	- Alternative to migrate from the previous version 5.4 to 5.5
		(SQLite3 to MySQL)
		sqlite3 serverdata/tablondb.sqlite .dump | ./sqlite2mysql.py > tablondb.sql
		mysql --user=tablon --password=....... tablon < tablondb.sql


Configure Tablón
---------------------------------------------------------------------

Change the line 336 of client.py to set the full domain name of the server
that executes Tablón.

All the Tablón configuration is done in the file server.py
(this is the main script)

For our course, we usually use the same file for the different
environments, the code has several parts that are dependent of the current hostname.

In that file, there is a class called CONFIG, it contains the configuration
attributes: server name, ip, port, time config, admin list...

Also this file contains the code to create the queues.
Currenly, there are four types (four python classes):
	tablon.LocalOpenMPQueue
	tablon.LocalMPIQueue
	tablon.LocalMPIOpenMPQueue
	tablon.SlurmQueue


There could be as many queues as needed, the arguments of the constructor are:
the name of the queue, the makefile used to compile the user code and other
parameters such as: the maximum number of threads/processes, timewall...

Example of a OpenMP queue:

q_omp = tablon.LocalOpenMPQueue('openmp', makefile='Makefile_openmp',
	threads=16,timewall=60*2, description="OpenMP queue")


In addition to queues, Tablón has leaderboards. A leaderboard is associated
to a queue and it is used to rank the student solutions to a given problem.
The leaderboards are used in the same way as the queues, they have also
a name. Each leaderboard has several phases, in each phase Tablón executes
the student program with a set of the execution parameters and then it
checks the output of the program. 


Example of the creation of a leaderboard:

lb_ejemplo = q_omp.addLeaderboard('Ejemplos',isopen=False)
lb_ejemplo.addPhase({'nprocs':1,'threads':14,'arguments':'Input1 Input2 Input3'},[0,12,-1])
lb_ejemplo.addPhase({'nprocs':1,'threads':14,'arguments':'BiggerInput1 BiggerInput2'},[1234,12345])



User codes and compilation
---------------------------------------------------------------------

The user codes are stored in the "usercodes/src" folder, the filename is the
request id. Tablón modifies the code adding a comment with the request info
and it also adds a include to the header "cputilstablon.h". This header
is needed for OpenMP programs because it replaces the open_set_num_threads
function so the number of threads selected by Tablón has priority over the
number of threads inside the rest of the code. 

Moreover, Tablón will also copy in the "usercodes/src" folder the compilation
Makefiles and other header files defined in the queues. This files are copied
each time the server is started.

The user codes are compiled using the Makefile of the current queue.





User creation
---------------------------------------------------------------------

The users are created using the script useradmin.py.
They can be created one by one or by using a list.
If this script do not receive the password, it will generate it
randomly (see examples in the create_users.sh script).



File structure
---------------------------------------------------------------------

* client.py:
	Client program code. This python program will only work in the server.
	The users should download the client using the web interface.
	The client that is downloaded from the web is a modified version of
	this one that also includes the code of the srp library.

* Makefile
	It is used to clean up the installation, backups...

* server.py
	Main server code

* stop.sh
	Restarter stop script

* test_codes
	Folder with test codes

* user_list.txt
	User list. grupo01, grupo02... used by the script create_users.sh

* create_users.sh
	Script to create users, it has calls to useradmin.py.

* makefiles
	Folder with the queues makefiles, they are then copied to usercodes/src by the queues

* reiniciador.py
	Restarter script

* tablon
	Main source folder

* useradmin.py
	Script to create or delete users

* httpdocs
	Directory with the web static files

* README.md
	This file

* run.sh
	Restarter run script

* serverdata
	Folder with the database and leaderboard info

* ssl
	Folder with the ssl key

* templates
	HTML templates 

* usercodes
	Folder with the user codes (usercodes/src) and the binary user programs (usercodes/bin)





Files and database
---------------------------------------------------------------------

The information of users, requests, and leaderboards is stores in a
database. The current version has been tested with MySQL as database 
server.

You can use the script ScriptsSQL/db_createStructure.sql to create the
structure of the database.

The folder "serverdata" contains copy of some information also stored in 
the database. The use of this information is deprecated. It is still there
for debugging purposes.

To modify the users, you should use the useradmin.py script that creates
the SRP keys for the users. The leaderboard files can be modified manually
(for example to remove a non-valid user execution).


Reiniciador
---------------------------------------------------------------------

Tablón can be started just by calling ./server.py, but sometimes it
can get frozen when executing some code. To solve this, we use the
Reiniciador (restarter) script.

The restarter stops and runs again Tablón each hour.

The restarter script has to be started using the ./run.sh script,
this scripts calls nohup and saves the process pid in a txt file.
The ./stop.sh script reads the pid and kills Tablón and the restarter.

Moreover, the restarter script, checks if the server.py file has changed
to also restart the Tablón.





Reference times in the leaderboards
---------------------------------------------------------------------

The admin users (defined in a list in the server.py file) can use the
option -r (--reference) when sending executions to a leaderboard.
Using this option, instead of showing the user name, Tablón shows "reference"
as the name and it is show in the ranking list, but it does not count as a normal
user in the ranking.




CSS and javascript
---------------------------------------------------------------------

The css style is: http://purecss.io/

Moreover, we use several well-know javascript libraries: jquery, moment, highlight. 

The .css and .js files are located in CDNs (Content distribution network).




How does Tablón work?
---------------------------------------------------------------------

Tablón uses the basic BaseHTTPServer that is included in the python distribution.
It is an unique standalone application.

Tablón listens from a port (80 by default) to server both the web pages and
the execution requests. The web pages are served by normal HTTP GET requests.
The user execution requests are HTTP CONNECT requests used to send the
user source code. We use our own protocol, first we authenticate the user
using the SRP library, then we serialize a python object with the user code
and the execution parameters (queue, processes, threads, ...)

The received user source code is stored in a file and a new execution request
is created in the appropriate execution queue. Tablón has an executionmanager
that keeps a execution queue. It compiles and stars the programs using
several executionqueue objects. A executionqueue implements the same interface
so the executionmanager can use it but each of them implements different
types of queues (Sequential, OpenMP, MPI, and Slurm).


