#!/usr/bin/python2
# -*- coding: utf-8 -*-

################################################################################
#
# Servidor Tablon - Cliente
# Asignatura Computacion paralela
# Javier Fresno, 2015
#
################################################################################


# Imports
import sys
import binascii
import pickle
import re
import socket
from optparse import OptionParser
import getpass
import hashlib


# REPLACE SRP
sys.path.append("tablon")
from _pysrp import *
# REPLACE SRP


# CLIENT VERSION
CLIENT_VERSION = '1.7'

DEFAULT_PORT = 80
OTHER_PORT = 8080


# MAX SIZE: This is checked again in the server
MAX_PROGRAM_SIZE = 15000 * 10 # This is 10 times the size of mpi version reference version (curso 1314)

#
# Class for the Secure session
#
class ClientSession():

	# Constructor
	def __init__(self):

		self.connected = False
		self.error = None

	# Connect and authenticate
	def connect(self,host,port,user,password):


		print ('Conecting ...')

		# 1. Connect the socket
		try:
			self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

			# Try to connect either to the given port or to other (80)
			try:
				self.socket.connect((host, port))
				print ('Connected to %s@%s:%d' % (user,host,port))
			except:
				if not OTHER_PORT:
					raise
				port = OTHER_PORT
				self.socket.connect((host, port))
				print ('Connected to %s@%s:%d' % (user,host,port))

			self.wfile = self.socket.makefile('w+',bufsize=0)
			self.rfile = self.socket.makefile('r+')

		except socket.error:
			self.error = "Error connecting to server"
			return


		# 2. Try to authenticate
		try:

			# Send start message
			self.wfile.write('CONNECT /launcher HTTP/1.0\n\n')

			# Send version
			self.wfile.write(CLIENT_VERSION + '\n')

			# Get server ok
			ok = self.rfile.readline().strip()
			if ok != 'OK':
				print ok
				sys.exit(-1)

			# Authenticate user
			usr = User(user, password, hash_alg=SHA256)
			uname, A = usr.start_authentication()
			self.wfile.write(uname+'\n')
			self.wfile.write(binascii.b2a_hex(A)+'\n')

			# Get the challenge
			s = binascii.a2b_hex(self.rfile.readline().strip())
			B = binascii.a2b_hex(self.rfile.readline().strip())

			# Process challenge	and send response
			M = usr.process_challenge(s, B)
			if M is None:
				raise Exception()
			self.wfile.write(binascii.b2a_hex(M)+'\n')

			# Get the session
			HAMK = binascii.a2b_hex(self.rfile.readline().strip())
			usr.verify_session(HAMK)

			# Check authentication
			if usr.authenticated():
				print ("Successful authentication")
				self.connected = True
			else:
				raise Exception()

			# Get the session key
			self.key = usr.get_session_key()

		except:
			self.error = "Error authentication to server failed"
			return	

	# Send the request
	def send(self,request):

		# Msg
		print("Sending request")

		# Marshalling and send of the request
		data = pickle.dumps(request)
		self.wfile.write(str(len(data))+'\n')
		self.wfile.write(data)

		# Compute the hash of the file and our key
		m = hashlib.sha256()
		m.update(data)
		m.update(self.key)
		self.wfile.write(m.hexdigest()+'\n')

		# Get the returned message
		return self.rfile.read()
		self.socket.close()

	# Close the socket
	def close(self):
		self.socket.close()






# Class that contains a request for the execution of a program
class ExecutionRequest():

	# Constructor
	def __init__(self,options):

		# Get the needed options
		self.nprocs = options.nprocs
		self.threads = options.threads
		self.program = base=os.path.basename(options.program)
		self.arguments = options.args
		self.queue = options.queue
		if options.reference != None:
			self.reference = True
			self.reference_name = options.reference
		else:
			self.reference = False
		self.timewall = options.timewall
		self.cl1 = options.cl1

		# Read the file
		with file(options.program) as f:
			self.content = f.read()

		# Copy kernels (if any)
		self.copy_kernels(options.program)
		
		if len(self.content) > MAX_PROGRAM_SIZE:
			print ("Program too big")
			sys.exit(-1)


	# Search for all cuda kernels
	def copy_kernels(self,programpath):

		regex = re.compile('^\s*[#]include\s*["](kernel.*[.]cu)["]',re.MULTILINE)
		includes = re.finditer(regex, self.content)

		# Iterate over the kernels
		for include in includes:
			self.copy_kernel(include.group(0),include.group(1),programpath)




	# Search for a cuda kernel and inserts it on the content
	def copy_kernel(self,kernelinclude,kernelname,programpath):

		print "Including kernel %s" % kernelname

		# Check if there is a kernel.cu in the same folder
		if os.path.dirname(programpath) == '':
			kfile = kernelname
		else:
			kfile = os.path.dirname(programpath) + '/' + kernelname
		if not os.path.exists(kfile):
			print "Kernel %s not found" % kernelname
			sys.exit()

		# Read the kernel
		with file(kfile) as f:
			kcontent = f.read()

		# Add comments
		kcontent = '//--------- KERNEL ---------\n' + kcontent + '//--------- KERNEL ---------\n'

		# Escape the content in the kernel.cu
		# kcontent = kcontent.encode("string_escape")
		self.content = self.content.replace(kernelinclude,kcontent)







# Parse the input arguments
###############################################################################

parser = OptionParser(usage='usage: %prog [options] program.c/.cu [program args]')

# Default
parser.set_defaults(port=DEFAULT_PORT)
parser.set_defaults(server='myserver.mydomain')
parser.set_defaults(nprocs=1)
parser.set_defaults(threads=1)
parser.set_defaults(reference=None)
parser.set_defaults(timewall=None)
parser.set_defaults(cl1=None)

# Group server
parser.add_option('-u','--user')
parser.add_option('-x','--password')
parser.add_option('-s','--server')
parser.add_option('-p','--port',type="int")
parser.add_option('-n','--nprocs',type="int")
parser.add_option('-t','--threads',type="int")
parser.add_option('-q','--queue')
parser.add_option('-r','--reference')
parser.add_option('-w','--timewall',type="int",help='Timewall in seconds')
parser.add_option('-c','--cl1',help='Flag para gestion cache/shared')

(options, args) = parser.parse_args()

if len(args) == 0:
	parser.print_help()
	sys.exit(-1)

if not options.user:
	print ("Missing user name")
	sys.exit(0)

if not options.password:
	options.password = getpass.getpass('Write your password: ')

if options.nprocs < 1:
	print ("Wrong number of processes")
	sys.exit(-1)

if options.threads < 1:
	print ("Wrong number of threads")
	sys.exit(-1)



options.program  = args[0]
options.args = ' '.join(args[1:])


if not options.program.endswith('.c') and not options.program.endswith('.cu') and not options.program.endswith('.cpp'):
	parser.print_help()
	print ('Program must end with .c, .cpp or .cu')
	sys.exit(-1)

# Check file
if not os.path.exists(options.program):
	parser.print_help()
	print ('Program do not exists')
	sys.exit(-1)



# Create the session
cs = ClientSession()
cs.connect(options.server,options.port,options.user,options.password)

# Exit if not connected
if not cs.connected:
	print (cs.error)
	sys.exit(-1)

# Create the request with the options
er = ExecutionRequest(options)

# Send the request and get the sever msg
msg = cs.send(er)
print (msg)





















