#!/usr/bin/python
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################


#
# Import the tablon module
#
import tablon


# For hostname
import socket

#
# This class is passed to the tablon server
#
class CONFIG:

	last_request = 1

	# Just copy the hostname in the CONFIG
	hostname = socket.gethostname()

	# If the hostname is not one of the machines we are at the test host.
	if hostname not in ['myserver']:
		hostname = 'test'

	# Tablón version
	# It is important because the .js and .css files include the
	# name of the version in the url has a get parameter to
	# avoid caching when a new version is released
	version = '5.6'

	# Where to serve (by default)
	host='myserver.mydomain'

	# Change the host part on the hostname 
	if hostname != 'test':
		host =  hostname + ',mydomain'

	# Main color
	color = '#40C350' # Verde prueba

	# Color & favicon For a particular macine
	if hostname == 'nooo':
		color = '#1f8dd6' # Azul
		favicon = 'favicon_azul.ico'

	# Port for both sending files and web server
	port=8080

	# This is the real port in case we are redirecting the connections
#	real_port=80
	
	# Port for the https server
	sport=4443

	# MySQL information
	bd_host = "localhost"	# Change to your database info
	bd_user = "tablon"	# Change to your database info
	bd_pass = "foo"		# Change to your database info
	bd_name = "tablon1617"	# Change to your database info
	# Show debug messages
	verbose=False

	# HTTP frontend number of threads
	# The frontend handles web and launch request.
	# If this is more than 1, it runs another implementation of the HTTP server
	# that uses a pool of threads to allow simultaneous user requests.
	frontend_threads = 4 

	# Timeout for http connections
	http_timeout = 15
	if hostname == 'test':
		http_timeout = 60

	# Supported client version
	client_version = '1.7'

	# Send a compressed gzip auto executable python script
	# It has half the size and it is harder to hack.
	compress_client = True

	# Max size of the code
	max_program_size = 15000 * 10 # This is 10 times the size of my mpi version

	# The minimun time between user requests in seconds
	time_between_requests = 20
	if hostname == 'test':
		time_between_requests = 2

	# The minimun time between user requests in seconds
	# If they have used all the quota
	# (0 means no request are permitted)
	time_between_requests_no_quota = 60

	# Stop receiving requests X secons before the restart
	time_before_restart = 60

	# Number of programs in the main page
	requestnumber = 20

	# Size of the stdout and stderr
	maxstdoutsize = 2500 

	# Forbidden code in the programs
	forbidden=[
		'##',
		'fork',
		'exec',
		'popen',
		'fopen',
		'open',
		'setjmp',
		'remove',
		'rename',
		'system',
		'getenv',
		'MPI_File_open',
		'sys/syscall.h',
		'sys/stat.h',
		'fstream'
	]
	#	'remove[(]',
	#	'rename[(]',
	#	'system[(]',
	#	'FILE',

	# Admin users: they do not have time restrictions
	# and they can send a program to the leaderboard as reference
	# This user names may not exists
	#admins = ['javi','admin1','teacher1','teacher2']
	admins = ['t01','t02','t03','t04','t05']

	# Enable the admin pages
	# It uses https secure connections
	# but if not used it is better to disable it
	adminpages = True

	# Users and passwords of the admin pages
	adminpass = [('admin','foo')]  # Change to your own password
	if hostname == 'test':
		adminpass.append(('test','test'))

	# Add code to the user source
	addtosrc = '#include "cputils.h" // Added by tablon';

	# Expire html content, used for files such as css or js (in hours)
	expire_html = 10


#
# List of extra files to compile
# We add them to any queue so they can be copied
#
extrafiles = [('cputils_2017.h','cputils.h'),'cputilstablon.h']
# Using a tuple, we can change the name
if CONFIG.hostname == 'machine1':
	extrafiles = ['cputils.h',('cputilstablon_m1.h','cputilstablon.h')]

# The previous line copies the cputils.h file from the makefile folder
# to the compile folder. It also copies the cputilstablon_m1.h and
# rename it to cputilstablon.h

if CONFIG.hostname == 'machine2':
	extrafiles = ['cutil32.lib','libcutil.a','libcutilD.a','cutil.h','cputils.h',('cputilstablon_m2.h','cputilstablon.h')]




#
# Create the queues
#

#
# IMPORTANT NOTE !!
# Do not create queues in the local machine (Tablon server). 
# It is a potential security risk.
#
# Do create queues that execute in backend machines where the home directory
# of the tablon user contains only the test files for the queue and the users
# executables transfered by tablon.
#


# Slurm Sequential queue
q_seq = tablon.SlurmQueue('sequential',
	extensions=['.c','.cpp'],
	makefile='Makefile_sequential',
	threads=1, 
	slurm_queue='slurm_partition_name', 
	slurm_nodes=['hydra'], 
	remote_binpath="/home/tablon/sandbox",
	timewall=60,
	extrafiles=extrafiles,
	description='Sequential queue for testing')

if True:
	# Leaderboard suma (to test the leaderboard system)
	lb_suma = q_seq.addLeaderboard('sum', isopen=True,
		description=u"The program receives several inputs as arguments and it should return the sum.")

	# Phases (tests)
	lb_suma.addPhase({'arguments':'1 1'},'2')       # 1 + 1 = 2
	lb_suma.addPhase({'arguments':'10 10 10'},'30') # 10 + 10 + 10 = 30
	lb_suma.addPhase({'arguments':'1 2 9'},'12')    # 1 + 2 + 9 = 12


# Slurm OpenMP queue
q_omp = tablon.SlurmQueue('openmp',
	isopen=False,
	extensions=['.c','.cpp'], 
	makefile='Makefile_openmp',
	threads=24, 
	slurm_queue='slurm_partition_name', 
	slurm_nodes=['node1'], 
	remote_binpath="/home/tablon/sandbox",
	timewall=60*2,
	extrafiles=extrafiles,
	description='OpenMP queue that execute in node1')


# Leaderboard openmp
lb_openmp = q_omp.addLeaderboard('openmplb',isopen=False,
	description=u"Leaderboard for an example program using the OpenMP queue")

# Phases
lb_openmp.addPhase({'nprocs':1,'threads':12,'arguments':'test1.data'},'345')       
lb_openmp.addPhase({'nprocs':1,'threads':12,'arguments':'test2.data'},'12')       
lb_openmp.addPhase({'nprocs':1,'threads':12,'arguments':'test3.data'},'123')       
lb_openmp.addPhase({'nprocs':1,'threads':4,'arguments':'test3.data'},'123')       


## Slurm MPI queue
q_mpi = tablon.SlurmQueue('mpi',extensions=['.c'], makefile='Makefile_mpi',
	nprocs=20, slurm_queue='slurm_partition_name', slurm_nodes=['node1','node2','node2','node4'], remote_binpath="/home/tablon/sandbox",
	timewall=60*2,
	description='MPI queue that execute in up to four nodes')

# Leaderboard mpi
lb_mpi = q_mpi.addLeaderboard('mpilb', isopen=False,
	description=u"Leaderboard for an example program using the MPI queue")

lb_mpi.addPhase({'nprocs':20,'threads':1,'arguments':'test1.data'},'345')       
lb_mpi.addPhase({'nprocs':20,'threads':1,'arguments':'test2.data'},'12')       
lb_mpi.addPhase({'nprocs':8,'threads':1,'arguments':'test3.data'},'123')       
lb_mpi.addPhase({'nprocs':20,'threads':1,'arguments':'test3.data'},'123')       


# Slurm CUDA queue
q_cuda = tablon.SlurmQueue('cuda',extensions=['.cu'], 
	isopen=False,
	makefile='Makefile_cuda',
	slurm_queue='slurm_partition_name', 
	slurm_nodes=['nodeGPUs'], 
	ngpus=1,
	remote_binpath="/home/tablon/sandbox",
	timewall=60*2,
#	extrafiles=['cutil32.lib','libcutil.a','libcutilD.a','cutil.h',('cputilstablon_m2.h','cputilstablon.h')],
	description='CUDA queue that execute in the GPUs of a node')

# Leaderboard CUDA
lb_cuda = q_cuda.addLeaderboard('cudalb', isopen=False,
	description=u"Leaderboard for an example program using the CUDA queue")

lb_cuda.addPhase({'nprocs':1,'threads':1,'arguments':'test1.data'},'345')       
lb_cuda.addPhase({'nprocs':1,'threads':1,'arguments':'test2.data'},'12')       
lb_cuda.addPhase({'nprocs':1,'threads':1,'arguments':'test3.data'},'123')       
lb_cuda.addPhase({'nprocs':1,'threads':1,'arguments':'test3.data'},'123')       




#
# Create and run the server
#
tablon.run(CONFIG)

