#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
#
# Servidor Tablon - ExecutionRequest
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################


# Imports
import os
import time
import re
import utils


#
# Status Enum
#
class Status:
	QUEUED = 1
	COMPILING = 2
	DEPLOYING = 3
	WAITING = 4
	RUNNING = 5
	FINISHED = 6
	CANCELED = 7
	ERROR = 9



#
# Class used to execute a program
#
class ExecutionRequest:

	# This is not a constructor.
	# This takes a received request by the client and adds all the missing fields
	def init(self):

		# Add the Code variables
		self.hash = utils.code_hash(self.content)
		nloc, ccn, ntok = utils.code_stats(self.program,self.content)
		self.lines = nloc
		self.tokens = ntok
		self.complexity = ccn

		# Add code
		if CONFIG.addtosrc:
			self.content = CONFIG.addtosrc + "\n" + self.content

		# Add user info
		info = "// User: %s@%s \n" % (self.user,self.address)
		info += "// %s \n" % str(self)
		info += time.strftime("// %b %d %Y %H:%M:%S\n")
		self.content = info + self.content



	#
	# Check that the fields have the correct types
	#
	def sanitize(self):


		# Set the attributes
		self.rid = None
		self.status = Status.QUEUED
		self.message = ''
		self.output = ''
		self.time = 0
		self.timestamp = utils.get_timestamp()

		# Leaderboard variables
		self.leaderboard = False
		self.lb_time = 0.0
		self.lb_phase = 0

		# Code info variables
		self.hash = ''
		self.lines = 0
		self.tokens = 0
		self.complexity = 0


		# The client sends these:
		# self.nprocs
		# self.threads
		# self.program
		# self.arguments
		# self.queue
		# self.reference
		# self.reference_name
		# self.timewall

		try:

			self.nprocs = int(self.nprocs)
			self.threads = int(self.threads)
			self.program = self.program.decode('utf8', 'replace')
			self.arguments = self.arguments.decode('utf8', 'replace')
			self.queue = self.queue.decode('utf8', 'ignore')
			if self.cl1 != None:
				self.cl1 = self.cl1.decode('utf8', 'replace')
			self.reference = bool(self.reference)
			if self.reference:
				self.reference_name = self.reference_name.decode('utf8', 'replace')
			if self.timewall != None:
				self.timewall = int(self.timewall)

		except Exception as e:

			#print e
			#import traceback
			#print traceback.format_exc()

			return False

		return True


	def setLeaderboard(self):
		self.leaderboard = True
		self.nprocs = 1
		self.threads = 1
		self.arguments = ''

	#
	# Check if the request has finished
	#
	def finished(self):
		if self.status == Status.ERROR or self.status == Status.FINISHED:
			return True
		else:
			return False

	#
	# Check extension
	#
	def extension(self):
		return os.path.splitext(self.program)[1]

	#
	# Return the source path
	#
	def src_name(self,path=None):
		if path:
			path = path + "/"
		else:
			path=''
		return "%sp%06d%s" % (path,self.rid,self.extension())

	#
	# Return the binary path
	#
	def bin_name(self,path=None):
		if path:
			path = path + "/"
		else:
			path=''
		return "%sp%06d" % (path,self.rid)


	#
	# Check if the code has something fobidden
	#
	def check_forbidden(self):

		# Remove the comments
		code = self.comment_remover()

		# Check fo the errors
		for i, l in enumerate(code.split('\n')):
			for fc in CONFIG.forbidden:
				if re.search(fc,l):
					return (i+1,l)

		return (None,None)

	
	#
	# Remove comments from the code to check for the forbiden code.
	# http://stackoverflow.com/questions/241327/python-snippet-to-remove-c-and-c-comments
	# I modified to keeps the spaces so the lines do not change
	#
	def comment_remover(self):

		# Get the text
		text = self.content

		# Defien the replacer function
		def replacer(match):
			s = match.group(0)
			if s.startswith('/'):
				return " " + "\n" * s.count('\n') # note: a space and not an empty string
			else:
				return s

		# Create the regexp
		pattern = re.compile(
			r'//.*?$|/\*.*?\*/|\'(?:\\.|[^\\\'])*\'|"(?:\\.|[^\\"])*"',
			re.DOTALL | re.MULTILINE
		)

		# Execute it
		return re.sub(pattern, replacer, text)


	#
	# To string
	#
	def __str__(self):
		return "ExecutionRequest[P:'%s',P:%d,T:%d,args:'%s',q:'%s']" % (self.program,self.nprocs,self.threads,self.arguments,self.queue)






# I think this was necessary to unmarshall the user request in
# in the launchserver class

#
# Add the ExecutionRequest request to the main module
#
import sys
setattr(sys.modules['__main__'], 'ExecutionRequest', ExecutionRequest)


