#!/usr/bin/python
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon - Execution thread
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################

#
# Logging
#
import logging
logger = logging.getLogger('tablon')

#
# Instance to tablon
#
import sys
tablon = sys.modules['tablon'] 

# Imports
import subprocess
import threading
import time


#
# An execution thread is a wrapper of a subprocess that gets the elapsed time
#
class ExecutionThread(object):


	def __init__(self,cmd,cwd=None,env=None,shell=False,slurm=False):

		self.start = time.time()
		self.end = 0
		self.slurm = slurm
		self.slurm_error = None
		self.process = subprocess.Popen(cmd, cwd=cwd, env=env, shell=shell, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
		self.returncode = None

		# Create the thread to watch for the command
		self.thread = threading.Thread(target = self.thread_method)
		self.thread.start()


	def thread_method(self):


		# If this is a slurm execution, the time starts when slurm
		# granted the job allocation
		if self.slurm:

			# Read from stdout until slurm had granted the job
			while True:
				line = self.process.stdout.readline()
				if line.startswith('salloc: Required node not available'):
					self.slurm_error = 'Node not available'
					self.kill()
					break
				if line.startswith('salloc: Granted job allocation'):
					self.start = time.time()
					break
				if line == '':
					break

		while(self.process.poll() == None):
			time.sleep(0.01)
		self.end =  time.time()

	def communicate(self,fullout=False):
		self.thread.join()
		self.returncode = self.process.returncode

		if self.slurm_error != None:
			out = self.slurm_error
		else:
			if fullout:
				out, err = self.process.communicate()
			else:
				out = self.process.stdout.read(CONFIG.maxstdoutsize)

		try:
			out = out.decode('utf-8','ignore')
		except:
			out = "Wrong characters"

		return out, None

	def elapsed(self):
		if self.poll() == None:
			return time.time() - self.start 
		else:
			self.thread.join()
			return self.end - self.start
	

	def poll(self):
		return self.process.poll()



	def kill(self):
		#self.process.terminate() # send sigterm
		self.process.kill()      # send sigkill


