#!/usr/bin/python
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon - Frontend
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################


#
# Logging
#
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('tablon')

# The http server
from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler

from webserver2 import WebServer
import webpages
import webpagesstats
if CONFIG.adminpages:
	import webpagesadmin
from launchserver import LaunchServer

import socket

# Threads
import threading

import tablondb

import ssl





#
# The frontend is a HTTPServer with handler that manages the web requests (GET)
# and the launch requests (CONNECT)
#
class Frontend:


	def __init__(self):
		self.running = False
		self.srunning = False

	# Run the frontend
	def run(self):

		try:

			#
			# Select the HTTP Server implementation
			#
			if CONFIG.frontend_threads == 1:
				logger.info("Using the basic HTTP Server with 1 thread")
				SelectedHTTPServer = HTTPServer
			else:
				logger.info("Using the multithreaded HTTP Server")
				from threadpoolmixin import ThreadPoolMixIn
			
				# Create the new server class
				class SelectedHTTPServer(ThreadPoolMixIn, HTTPServer):
					def __init__(self, server_address, RequestHandlerClass):
						self.numThreads=CONFIG.frontend_threads
						HTTPServer.__init__(self, server_address, RequestHandlerClass)


			#
			# Create and start the server
			#
			logger.info("Starting HTTP frontend")
			self.server = SelectedHTTPServer((CONFIG.host,CONFIG.port), FrontendHandler)
			self.server.using_ssl = False 
			self.thread = threading.Thread(target=self.run_thread)
			self.running = True
			self.thread.start()

			#
			# Create and start the https server
			#
			if CONFIG.adminpages:
				logger.info("Starting HTTPS frontend")
				self.sserver = HTTPServer((CONFIG.host,CONFIG.sport), FrontendHandler)
				self.sserver.socket = ssl.wrap_socket(self.sserver.socket, certfile='ssl/server.pem', server_side=True)
				self.sserver.using_ssl = True 
				self.sthread = threading.Thread(target=self.run_sthread)
				self.srunning = True
				self.sthread.start()

			return True

		except socket.error as e:
			logger.error("Socket Error %s" % e.args[1])
		except:
			logger.error("Unable to start HTTP frontend")

		return False

	# Stop the server
	def start_stop(self):
		self.server.shutdown()
		if CONFIG.adminpages:
			self.sserver.shutdown()

	def run_thread(self):
		self.server.serve_forever()
		self.running = False
		logger.info("HTTP frontend closed")


	def run_sthread(self):
		self.sserver.serve_forever()
		self.srunning = False
		logger.info("HTTPS frontend closed")

	def is_running(self):
		return self.running and self.srunning


#
# The handler for the frontend http server
#
class FrontendHandler(BaseHTTPRequestHandler,WebServer,LaunchServer):

	def __init__(self,request, client_address, server):
		WebServer.__init__(self)
		BaseHTTPRequestHandler.__init__(self, request, client_address, server)

	# Message logger
	def log_message(self, format, *args):
		user = tablondb.get_user_from_ip(self.client_address[0])
		logger.info("HTTP from user '%s' %s [%s] %s" % (user,self.address_string(),self.log_date_time_string(),format%args) )

	
	def setup(self):
		BaseHTTPRequestHandler.setup(self)
		self.rfile._sock.settimeout(CONFIG.http_timeout)







