#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
#
# Servidor Tablon - LeaderBoard
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################


#
# Logging
#
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('tablon')


#
# Instance to the tablon module
#
import sys
tablon = sys.modules['tablon'] 

# Imports
from executionrequest import Status
import re
import tablondb
import os
import codecs


#
# Leaderboard Phase: a test the user program has to pass
#
class LeaderBoardPhase:

	def __init__(self,override_request,results,resulterror):

		self.override_request = override_request
		if type(results) is not list:
			self.results = [results];		
		else:
			self.results = results
		self.resulterror = resulterror


	# Change the parameters and other attributes of the request object
	def override(self,request):
	
		for key in self.override_request:
			#print "override ", key, self.override_request[key]
			setattr(request, key, self.override_request[key])


	def check(self,request):
		
		ok, msg = self.check_results(request)
		if not ok:	
			request.status = Status.ERROR
			request.message = "Error in result %s" % msg
			return False


		t = self.get_time(request)
		if t == None:
			request.status = Status.ERROR
			request.message = "Time not found"
			return False
		request.lb_time += t

		return True


	def check_results(self,request):

		try:

			regex = re.compile('^Result:(.*)$',re.MULTILINE)
			values = re.finditer(regex, request.output)

			# Iterator to list
			values = list(values)

			# Check len
			if len(values) != len(self.results):
				return (False, ": invalid number of results")

			for i, v in enumerate(values):
				value = v.group(1).strip()
				if not self.check_result(i,value):
					return (False, ": result not valid")

		# If something is wrong, it is false
		except ValueError:
			return (False, ": value error")
		except:
			return (False, ": unexpected error")

		return (True,None)


	def check_result(self,i,value):

		if isinstance(self.results[i], int):
			value = int(value)
			return abs(value-self.results[i]) <= self.resulterror
		elif isinstance(self.results[i], float):
			value = float(value)
			return abs(value-self.results[i]) <= self.resulterror
		elif isinstance(self.results[i], str):
			return value == self.results[i]

		return False
			


	def get_time(self,request):

		try:

			# Search for the time in the output
			regex = re.compile('^Time:(.*)$',re.MULTILINE)
			search = re.search(regex, request.output)

			if search:
				# Get the value
				value = search.group(1) 
				return float(value)

		except:
			pass
		
		return None
			

#
# A leaderboard
#
class LeaderBoard:

	# File to store the leaderboard
	FILE = 'serverdata/%s.lb'

	# Constructor
	def __init__(self,name,description='',isopen=True):
		self.name = name
		self.description = description
		self.phases = []
		self.items = []
		self.isopen = isopen
		self.load()
		self.sort()
		self.save()

		if isopen:
			tablon.leaderboards[0:0] = [self]
		else:
			tablon.leaderboards.append(self)




	def addPhase(self,override,result,resulterror=0):
		phase = LeaderBoardPhase(override,result,resulterror)
		self.phases.append(phase)

	def getPhase(self,idx):
		return self.phases[idx]

	def numPhases(self):
		return len(self.phases)



	#
	# Add a request into the leaderboard creating a LeaderBoardItem
	#
	def add(self,request):

		# Create and add a new item
		nitem = LeaderBoardItem(request)		
		self.items.append(nitem)
		tablondb.add_leaderboard(nitem)

		# sort
		self.sort()

		# save to file
		self.save()

	def sort(self):
		self.items = sorted(self.items, key=lambda i: (i.user, i.time))

	def save(self):

		directory = "serverdata"
		if not os.path.exists(directory):
			os.makedirs(directory)

		with codecs.open(self.FILE % self.name , 'w',"utf-8") as f:
			for item in self.items:
				f.write(item.toLine())

	def load(self):
		try:
			with codecs.open(self.FILE % self.name , 'r', "utf-8") as f:
				for line in f:

					nitem = LeaderBoardItem(line)
					self.items.append(nitem)
					# Remove completely
					if tablondb.initiated:
						tablondb.add_leaderboard(nitem)

		except :
			self.items = []


	# iterator: the array
	def __iter__(self):
		return iter(self.items)

	# len: the array
	def __len__(self):
		return len(self.items)



#
# An item of a leaderboard
#
class LeaderBoardItem:

	def __init__(self,request_or_line):

		if isinstance(request_or_line, str) or isinstance(request_or_line, unicode) :

			line = request_or_line.split(',')

			self.rid = int(line[0])
			self.user = line[1]
			self.program = line[2]
			self.queue = line[3]
			self.time = float(line[4])
			self.rtime = float(line[5])
			self.timestamp = int(line[6])

		else:

			request = request_or_line

			self.rid = request.rid
			if request.reference:
				self.user = 'reference'
				self.program = request.reference_name
			else:
				self.user = request.user
				self.program = request.program
			self.queue = request.queue
			self.time = request.lb_time
			self.rtime = request.time
			self.timestamp = request.timestamp


	def toLine(self):
		return "%d,%s,%s,%s,%f,%f,%d\n" % (self.rid,self.user,self.program,self.queue,self.time,self.rtime,self.timestamp)










