#!/usr/bin/python
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon - Users
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################

#
# Logging
#
import logging
logger = logging.getLogger('tablon')

#
# Import the database
#
import tablondb

#
# Import authentication library
#
from load_srp import *

# Other modules
import os
import csv
import binascii


# TODO: Add a row to the files with the items

#
# Class that keeps the loging info of a user
#
class User:

	# Create the User
	def __init__(self, name=None, password=None, quota=0):

		# This is to create a empty user to use the from Row method
		if not name and not password:
			return

		# Create the salt and vkey for the authentication
		salt, vkey = srp.create_salted_verification_key(name, password, hash_alg=srp.SHA256)

		# Save all the data
		self.name = name
		self.salt = salt
		self.vkey = vkey
		self.quota = quota

		# The stats
		self.requests = 0
		self.errors = 0
		self.usedquota = 0

	# Load an user from a csv row
	def fromRow(self,row):
		name, salt, vkey, quota = row
		self.name = name
		self.salt = binascii.a2b_hex(salt)
		self.vkey = binascii.a2b_hex(vkey)
		self.quota = int(quota)

		# This is not defined if we are executing the user admin
		try:
			if name in CONFIG.admins:
				self.admin = True
			else:
				self.admin = False
		except:
			pass

		# The stats
		self.requests = 0
		self.errors = 0
		self.usedquota = 0

	# Save the user as a csv row
	def toRow(self):
		name = self.name
		salt = binascii.b2a_hex(self.salt)
		vkey = binascii.b2a_hex(self.vkey)
		quota = str(self.quota)
		return [name,salt,vkey,quota]


	# Load user stats from a csv row
	def stats_fromRow(self,row):
		name, requests, errors, usedquota  = row
		self.requests = int(requests)
		self.errors = int(errors)
		self.usedquota = float(usedquota)


	# Save the user stats as a csv row
	def stats_toRow(self):
		name = self.name
		requests = str(self.requests)
		errors = str(self.errors)
		usedquota = str(self.usedquota)
		return [name,requests,errors,usedquota]


	def save_stats(self):
		self.userlist.save_stats()
	


	# Setter and getter from the database
	# ----------------------------------------------- #
	@property
	def usedquota(self):
		return tablondb.usedquota(self.name)


	# Increase the used quota
	def incquota(self, value):
		pass



	
#
# List with the users
#
class UserList:

	# File name
	FILE = 'serverdata/users.dat'
	FILESTATS = 'serverdata/userstats.dat'

	# Init the user list
	def __init__(self):
		self.users = []
		self.load()
		self.load_stats()


	# Save the users to a CSV
	def save(self):


		with open(self.FILE, 'w') as f:
			writer = csv.writer(f)
			for u in self.users:
				writer.writerow(u.toRow())

	# Load the CSV
	def load(self):

		self.users = []
		if os.path.isfile(self.FILE):
			with open(self.FILE, 'r') as f:
				reader = csv.reader(f)
				for row in reader:
					u = User()
					u.userlist = self
					u.fromRow(row)
					self.users.append(u)

		logger.info("Loaded user list: %d users" % len(self.users))


	# Save the stats to a CSV
	def save_stats(self):
		with open(self.FILESTATS, 'w') as f:
			writer = csv.writer(f)
			for u in self.users:
				writer.writerow(u.stats_toRow())


	# Load the CSV
	def load_stats(self):

		if os.path.isfile(self.FILESTATS):
			with open(self.FILESTATS, 'r') as f:
				reader = csv.reader(f)
				for row in reader:
					name = row[0]
					u = self.get(name)
					if u:
						u.stats_fromRow(row)


	# Get user
	def get(self,name):
		for u in self.users:
			if u.name == name:
				return u
		return None

	# Change quota
	def chquota(self,name,quota):
		for i, u in enumerate(self.users):
			if u.name == name:
				self.users[i].quota = quota


	# Create a new user
	def create(self,name,password,quota=0):


		# Remove previous user
		self.remove(name)

		# Create, append, and save the list
		user = User(name,password,quota)
		self.users.append(user)
		self.save()

		# Return the user
		return user


	# Remove a user
	def remove(self,name):
		for i,u in enumerate(self.users):
			if u.name == name:
				del self.users[i]
				self.save()
				return True
		return False


	# iterator: the array
	def __iter__(self):
		return iter(self.users)


	# len: the array
	def __len__(self):
		return len(self.users)

