#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
#
# Servidor Tablon - WebPages
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################


#
# Logging
#
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('tablon')

#
# Instance to the tablon module
#
import sys
tablon = sys.modules['tablon'] 

#
# Tablon imports
#
from webserver2 import webrequest, web, render_template, Context, error_page
import tablondb

#
# Other imports
#
import datetime
import re
import utils



@web.url('/ok')
def users_page():

	"""
	An OK json
	"""

	webrequest.content_type = 'application/json'

	return '{"isok":true,"version":"%s"}' % (tablondb.get_dbversion(),)




@web.url('/')
def root_page():

	"""
	The main page of the web
	"""
	
	context = Context()
	context['subtitle'] = u'Computación Paralela'
	context['menuselected'] = 'Home'

	context['requests'] = tablondb.show_request_list()
	context['restartat'] = tablon.restartat

	return render_template('root.html',context)





@web.url('/users')
def users_page():

	"""
	The users page
	"""

	context = Context()
	context['title'] = 'Users'
	context['menuselected'] = 'Users'
	context['users'] = tablondb.show_users()

	return render_template('users.html',context)





@web.url('/faq',cache=60*60)
def faq_page():
	
	"""
	The fag page
	"""

	context = Context()
	context['title'] = 'FAQ'
	context['menuselected'] = 'FAQ'

	return render_template('faq.html',context)





@web.url('/client',cache=60*60)
def client_page():

	"""
	Get a the client script file
	"""

	# Change the content type
	webrequest.content_type = 'application/octet-stream'

	# Read the client script, add the pysrp library and other changes
	with open('client.py','rb') as f:
		content = f.read()
	with open('tablon/_pysrp.py','rb') as f:
		cont_pysrp = f.read()
	content = re.sub("'localhost'","'%s'" % CONFIG.host, content)

	if hasattr(CONFIG,"real_port"):
		content = re.sub("DEFAULT_PORT = 80","DEFAULT_PORT = %d" % CONFIG.real_port, content)
	else:
		content = re.sub("DEFAULT_PORT = 80","DEFAULT_PORT = %d" % CONFIG.port, content)

	content = re.sub('sys[.]path[.]append[(]"tablon"[)]\nfrom _pysrp import [*]', cont_pysrp, content)

	# Compress if needed
	if CONFIG.compress_client:
		content = utils.gz_pack(content)

	# Send the client cache file
	return content




@web.url('/request')
def request_page():

	"""
	Page with the info of a particular request
	"""


	# Get the request id (if any)
	try:
		rid = int(webrequest.getParam('rid'))
	except:
		rid = None

	context = Context()
	context['title'] = u'No Request'
	if rid != None:
		rdb = tablondb.show_request(rid)
		if rdb:
			context['request'] = rdb			
			context['title'] = u'Request %05d' % rid


	return render_template('request.html',context)










@web.url('/tablon.css',ssl="Both")
def send_css():

	"""
	Tablon.css
	"""

	webrequest.content_type = 'text/css'
	webrequest.cachecontrol = True

	context = dict()
	context['color'] = CONFIG.color

	return render_template('tablon.css',context)





@web.url('/tablon.js',ssl="Both")
def send_js():

	"""
	Tablon.js 
	"""

	webrequest.content_type = 'application/javascript'
	webrequest.cachecontrol = True

	context = dict()

	return render_template('tablon.js',context)






@web.url('/leaderboards',cache=10)
def leaderboards_page():

	"""
	Page with all the leaderboards
	"""

	context = Context()
	context['title'] = u'Leaderboards'
	context['leaderboards'] = []
	context['menuselected'] = 'Leaderboards'

	for lb in tablon.leaderboards:
		items =  tablondb.show_leaderboard(lb.name)
		leaderboard = {'name':lb.name,'isopen':lb.isopen,'description':lb.description,'items':items}
		context['leaderboards'].append(leaderboard)

	return render_template('leaderboards.html',context)





@web.url('/queues',cache=60)
def queue_page():

	"""
	Page the with queues
	"""

	context = Context()
	context['title'] = 'Queues'
	context['menuselected'] = 'Queues'
	context['queues'] = tablon.onlyqueues

	return render_template('queues.html',context)



@web.url('/ajax_request_rows')
def ajax_request_rows():

	"""
	Ajax page to update the request page
	"""

	# Get the last request id (if any)
	try:
		rid = int(webrequest.getParam('rid'))
	except:
		rid = None

	# Send the html message
	context = dict()
	context['requests'] = tablondb.show_request_list(rid)

	return render_template('request_row.html',context)








