#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
#
# Servidor Tablon - WebPages for the admin
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################


#
# Logging
#
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('tablon')

#
# Instance to the tablon module
#
import sys
tablon = sys.modules['tablon'] 

#
# Tablon imports
#
from webserver2 import webrequest, web, render_template, AdminContext, Context
import tablondb


#
# Other imports
#
import re
import utils
import os
import uuid
import zipfile
import StringIO



#
# Array with the cookes of the logged admins
#
admincookie = []



@web.url('/admin')
def admin_redirect():

	"""
	Redirect to the https admin page from the regular http page 
	"""

	webrequest.response = 302

	# Check the original port 80 or 8080
	if webrequest.original_port == 80 or webrequest.original_port == 443:
		url = 'https://%s/admin' % (CONFIG.host)
	else:
		url = 'https://%s:%d/admin' % (CONFIG.host,CONFIG.sport)

	webrequest.send_header('Location', url )








def check_admin_cookie():

	"""
	Checks if the cookie given by the browser is in the list
	meaning the user is logged.
	Since we use a https server, this should be secure.
	"""

	cookie = webrequest.getCookie('tablonadmin')

	if cookie in admincookie:
		return True
	else:
		return False	



@web.url('/admin',ssl=True)
def admin_page():

	"""
	The admin page
	"""

	# Check the cookie
	if not check_admin_cookie():
		return admin_login_page()

	context = AdminContext()
	context['menuselected'] = 'Admin'
	context['numcookies'] = len(admincookie)
	context['leaderboards'] = tablon.leaderboards

	return render_template('admin.html', context)






@web.url('/admin/leaderboard',ssl=True)
def admin_leaderboard_page():

	"""
	The leaderboard page
	"""

	# Check the cookie
	if not check_admin_cookie():
		return admin_login_page()

	# Get the leaderboard name
	lbname = webrequest.getParam('lb')

	context = AdminContext()
	context['lb'] = next((lb for lb in tablon.leaderboards if lb.name == lbname), None)
	context['items'] = tablondb.show_leaderboard_admin(lbname)

	return render_template('adminleaderboard.html', context)



@web.url('/admin/leaderboardzip',ssl=True)
def admin_leaderboardzip():

	"""
	Zip with the leaderboard codes
	"""

	# Check the cookie
	if not check_admin_cookie():
		return admin_login_page()

	# Get the leaderboard name
	lbname = webrequest.getParam('lb')

	# Get leaderboard info from database
	lb = tablondb.show_leaderboard_admin(lbname)

	# Set webrequest variables
	webrequest.content_type = 'application/zip'
	webrequest.is_binary = True

	# Create the zip
	inMemory = StringIO.StringIO()
	zipf = zipfile.ZipFile(inMemory, 'w', zipfile.ZIP_DEFLATED)

	# Iterate adding the files
	for x in lb:

		if x["rid"] == None:
			continue

		extension = os.path.splitext(x['program'])[1]

		rid = x["rid"]
		name = "%s_%06d%s" % (x["user"],rid,extension) 
		content = "Error No Code"
		try:
			fname = "%sp%06d%s" % ('usercodes/src/',rid,extension)
			f = open(fname, "rw+")
			content = "// Leaderboard %s: pos %d\n" % (lbname,x["pos"]) 
			content += f.read().decode('utf-8', 'ignore')
		except:
			pass

		zipf.writestr(name, content.encode('utf-8'))

	# Close the zip
	zipf.close()

	# Return the value
	return inMemory.getvalue()



@web.url('/admin/leaderboarduser',ssl=True)
def admin_leaderboard_user_page():

	"""
	The leaderboard page of an user with this leaderboard requests
	"""

	# Check the cookie
	if not check_admin_cookie():
		return admin_login_page()

	lbname = webrequest.getParam('lb')
	user = webrequest.getParam('user')

	context = AdminContext()
	context['lb'] = next((lb for lb in tablon.leaderboards if lb.name == lbname), None)
	context['user'] = user
	context['items'] = tablondb.show_leaderboard_admin_user(lbname,user)

	return render_template('adminleaderboarduser.html', context)



@web.url('/admin/exit',ssl=True)
def admin_exit_page():

	"""
	Page to exit, it removes the cookie
	"""

	# Get the cookie
	cookie = webrequest.getCookie('tablonadmin')

	if cookie in admincookie:
		admincookie.remove(cookie)

	webrequest.send_header('Set-Cookie','tablonadmin=byebye; Path=/admin' )
	admin_redirect()






def admin_login_page():

	"""
	The login page for the admins
	"""

	context = AdminContext()
	context['title'] = 'Login'

	user = webrequest.getParam('user')
	password = webrequest.getParam('password')

	# If no user or password,
	# it is the first time we entered the page
	if user == None and password == None:
		context['message'] = ''
		return render_template('login.html', context)

	# Get the CONFIG user/pass
	userpass = next((i for i in CONFIG.adminpass if i[0] == user), None)

	# If no userpass, the user has enter something wrong
	if userpass == None:
		context['message'] = 'No user/password'

	# Else: Check the password
	else:
		
		# If the password is OK, send the cookie
		if userpass[1] == password:

			cookie = uuid.uuid4().hex
			admincookie.append(cookie)
			webrequest.send_header('Set-Cookie','tablonadmin=%s; Path=/admin' % cookie )
			admin_redirect()
			return

		# If not, Send mensage
		else:
			context['message'] = 'Wrong user/password'

	# Return the template
	return render_template('login.html', context)







@web.url('/admin/request',ssl=True)
def request_page():

	"""
	Request admin page that shows the code sent by the user
	"""

	# Check the cookie
	if not check_admin_cookie():
		return admin_login_page()

	# Get the request id (if any)
	try:
		rid = int(webrequest.getParam('rid'))
	except:
		rid = None

	context = AdminContext()
	context['title'] = u'No Request'
	if rid != None:
		rdb = tablondb.show_request(rid)
		if rdb:
			context['request'] = rdb			
			context['title'] = u'Request %05d' % rid

			try:
				extension = os.path.splitext(rdb['program'])[1]
				fname = "%sp%06d%s" % ('usercodes/src/',rid,extension)
				f = open(fname, "rw+")
				context['code'] = f.read().decode('utf-8', 'ignore')
			except Exception as e:
				context['code'] = str(e)
				

	return render_template('adminrequest.html',context)









