#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
#
# Servidor Tablon - WebPages Stats
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################


#
# Logging
#
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('tablon')


from webserver2 import webrequest, web, render_template, Context, error_page
import tablondb

import datetime

#
# Instance to the tablon module
#
import sys
tablon = sys.modules['tablon'] 

import re
import utils


@web.url('/stats')
def stats_page():

	"""
	This is the main stats page
	with links to the other pages
	"""

	# Try to load the module for the graphs
	try:
		import pygal
	except ImportError:
		return error_page(501,'I need pygal module')


	# Create a context with the data
	context = Context()
	context['title'] = 'Stats'
	context['menuselected'] = 'Stats'

	#
	# List of links
	#	
	context['statslinks'] = []

	# All time
	context['statslinks'].append( ('All time','/stats/requests') )

	# Today
	ini = utils.get_timestamp_day()
	end = ini + 86400
	context['statslinks'].append( ('Today','/stats/requests?ini=%d&end=%d' % (ini,end)) )

	# Last week
	ini = utils.get_timestamp_day() - 86400 * 7
	end = utils.get_timestamp_day() + 86400
	context['statslinks'].append( ('Last 7 days','/stats/requests?ini=%d&end=%d' % (ini,end)) )

	# TODO: OpenMP
	if None:
		ini = utils.get_timestamp_day(2015,2,24)
		end = utils.get_timestamp_day(2015,3,24)
		context['statslinks'].append( ('OpenMP','/stats/requests?ini=%d&end=%d' % (ini,end)) )

	# TODO: Add MPI
	# TODO: Add CUDA



	#
	# Leaderboardlinks
	#
	context['lblinks'] = []
	for lb in tablon.leaderboards:
		context['lblinks'].append( (lb.name,'/stats/leaderboard?lb=' + lb.name) )




	# Render the page
	return render_template('stats.html',context)








@web.url('/stats/requests',cache=60*5)
def stats_page():

	"""
	This is the page with stats for the requests:
	total number, graphs, types, errors...
	"""

	# Try to load the module for the graphs
	try:
		import pygal
	except ImportError:
		return error_page(501,'I need pygal module')


	# Create a context with the data
	context = Context()
	context['title'] = 'Requests stats'


	# Get the ini and end times
	try:
		ini = int(webrequest.getParam('ini'))
		end = int(webrequest.getParam('end'))
	except:
		ini = 0
		end = utils.get_timestamp()


	#
	# Get the ranges
	#
	context['ini'] = tablondb.show_stats_first_after_ts(ini)
	context['end'] = tablondb.show_stats_last_before_ts(end)


	# Get the total request number and used time
	totalrequests = tablondb.show_stats_total_requests(ini,end)
	context['numrequests'] = totalrequests['number']
	context['totaltime'] = totalrequests['time']


	# Pygal config
	config = pygal.Config()
	config.style=pygal.style.CleanStyle
	config.disable_xml_declaration=True
	config.width = 640 
	config.height = 480


	#
	# Requests by day
	#
	requests_day = tablondb.show_stats_request_byday(ini,end)
	chart = pygal.DateLine(config)

	days = []

	for day in requests_day:
		dt = day['date']
		days.append((dt, day['number']))

	chart.add('days',days)

	context['request_byday'] = chart.render(x_label_rotation=30, show_legend = False)


	#
	# Requests by hour
	#
	requests_hour = tablondb.show_stats_request_byhour(ini,end)
	chart = pygal.Bar(config)

	hours = []
	numbers = []
	for requests in requests_hour:
		hours.append(requests['hour'])
		numbers.append(requests['number'])

	chart.x_labels = map(str, hours)
	chart.add('Request',numbers)

	context['request_byhour'] = chart.render(show_legend = False)



	#
	# Requests by week day
	#
	requests_day = tablondb.show_stats_request_byweekday(ini,end)
	chart = pygal.Bar(config)

	days = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday']
	numbers = []
	for requests in requests_day[1:7]:
		numbers.append(requests['number'])
	numbers.append(requests_day[0]['number'])

	chart.x_labels = map(str, days)
	chart.add('Request',numbers)

	context['request_byweekday'] = chart.render(show_legend = False)



	#
	# Requests statuses
	#
	statuses = tablondb.show_stats_request_status(ini,end)
	chart = pygal.Bar(config)

	for status in statuses:
		chart.add(status['name'], status['number'])

	context['request_statuses'] = chart.render()



	#
	# Requests error types
	#
	statuses = tablondb.show_stats_error_types(ini,end)
	chart = pygal.Bar(config)

	for status in statuses:
		chart.add(status['message'], status['number'])

	context['error_types'] = chart.render()



	# Render the template
	return render_template('stats_requests.html',context)






@web.url('/stats/leaderboard',cache=60*5)
def stats_leaderboard_page():

	"""
	This is the page with stats for the requests:
	total number, graphs, types, errors...
	"""

	# Try to load the module for the graphs
	try:
		import pygal
	except ImportError:
		return error_page(501,'I need pygal module')


	# Get the leaderboard
	try:
		lbname = webrequest.getParam('lb')
		leaderboard = next((lb for lb in tablon.leaderboards if lb.name == lbname), None)
		if leaderboard == None:
			raise Exception
	except:
		return error_page(404,'Leaderboard do not exists')



	# Create a context with the data
	context = Context()
	context['title'] = 'Stats ' + lbname


	# Get the user info 
	users = tablondb.show_stats_leaderboard_plot(lbname)

	# Create the chart
	config = pygal.Config()
	config.disable_xml_declaration=True
	config.width = 640 
	config.height = 480
	config.x_label_rotation=20
	config.legend_font_size=7
	config.legend_box_size=5
	chart = pygal.DateLine(config)

	# Formatter to show the day only once
	def dateformatterlb(dt):
		current = dt.strftime("%Y-%m-%d")
		if current == dateformatterlb.last:
			current = ''
		else:
			dateformatterlb.last = current
		return current
	dateformatterlb.last = None

	# Formatters
	chart.x_value_formatter = dateformatterlb
	chart.value_formatter = lambda x: "%d" % abs(x)


	# Add the data
	for user in sorted(users.iterkeys()):
		points = [ (date,-pos) for date, pos in users[user] ]
		chart.add(user,points)


	# Change the colors
	import colorsys
	import copy
	style = copy.copy(pygal.style.CleanStyle)
	style.colors = []

	for i in range(0,360,360/len(users)):

		h = i/360.0
		l = 0.5
		s = 0.6

		hsl = (h,l,s)

		rgb = colorsys.hls_to_rgb(*hsl)
		rgb = tuple(int(i * 256) for i in rgb)
		html = utils.RGBToHTMLColor(rgb)

		style.colors.append(html)



	# Add the figure
	context['lbplot'] = chart.render(style=style)


	# Render the template
	return render_template('stats_leaderboard.html',context)









