#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<cuda.h>

#define	SEED	234767

cudaError_t error;
#define	CUDA_SAFE_CALL( call )	\
	if ( error=call ) { fprintf( stderr, "Error CUDA: %s\n", cudaGetErrorString( error ) ); \
	exit( EXIT_FAILURE );	}
#define	CUT_CHECK_ERROR( cad )	\
	if ( error=cudaGetLastError() ) { fprintf( stderr, "%s: %s\n", cad, cudaGetErrorString( error ) ); \
	exit( EXIT_FAILURE );	}


/* KERNEL FUNCTION */
__global__ void  sumaVectorVentana(int *A, int *B, int *C, int size, int windowSize){

	// 1. COMPROBAR QUE SOLO ACTUAN LOS HILOS NECESARIOS
	int posGlobal = threadIdx.x + blockIdx.x * blockDim.x;
	int w;
	for ( w=0; w+posGlobal<size; w+=windowSize ) {
		C[ posGlobal+w ] = A[ posGlobal+w ] + B[ posGlobal+w ];
	}
}


/* MAIN */
int main( int argc, char *argv[] )
{
	/* Leemos el argumento */
	if ( argc < 3 ) {
		fprintf( stderr, "Usage: %s <size> <tamBloque> <numIter>\n", argv[0] );
		exit( EXIT_FAILURE );
	}
	int size = atoi( argv[1] );
	int tamBloque = atoi( argv[2] );
	int numIter = atoi( argv[3] );
	
	if ( size % ( tamBloque * numIter) != 0 ) {
		fprintf( stderr, "Error parametros: <size> debe ser divisible por <tamBloque> * <numIter>\n" );
		exit( EXIT_FAILURE );
	}

	int numHilos = size / numIter;
	int numBloques = numHilos / tamBloque;
	int windowSize = size / numIter;

	/*Indicamos la GPU (DEVICE) que vamos a utilizar*/
	cudaSetDevice(0);

	/* Punteros a arrays en host y device */
	int *AHost, *BHost, *CHost;
	int *ADevice, *BDevice, *CDevice;
	
	/* Reservas de memoria HOST y DEVICE */
	AHost = (int*)malloc(sizeof(int) * size);
	BHost = (int*)malloc(sizeof(int) * size);
	CHost = (int*)malloc(sizeof(int) * size);
	CUDA_SAFE_CALL( cudaMalloc( &ADevice, sizeof(int) * size ) );
	CUDA_SAFE_CALL( cudaMalloc( &BDevice, sizeof(int) * size ) );
	CUDA_SAFE_CALL( cudaMalloc( &CDevice, sizeof(int) * size ) );

	/* Inicializacion e impresion inicial */
	printf("Inicializando arrays...");
	for(int i=0; i<size; i++){
		AHost[i] = -10;
		BHost[i] = i;
		CHost[i] = 0;
	}
	printf("\n");	

	/* Transferencia a memoria Device */
	CUDA_SAFE_CALL( cudaMemcpy(ADevice, AHost, sizeof(int)*size, cudaMemcpyHostToDevice) );
	CUDA_SAFE_CALL( cudaMemcpy(BDevice, BHost, sizeof(int)*size, cudaMemcpyHostToDevice) );
	CUDA_SAFE_CALL( cudaMemcpy(CDevice, CHost, sizeof(int)*size, cudaMemcpyHostToDevice) );

	/* Lanzamiento del kernel */
	dim3 grid(numBloques);
	dim3 bloque(tamBloque);
	printf("Lanzando. NumBloques: %d, tamBloque: %d: windowSize: %d\n", numBloques, tamBloque, windowSize );
	sumaVectorVentana<<< grid, bloque >>>(ADevice, BDevice, CDevice, size, windowSize);

	CUT_CHECK_ERROR( "Error en el kernel..." );
	cudaDeviceSynchronize();

	/* Transferencia a memoria Host */
	CUDA_SAFE_CALL( cudaMemcpy(CHost, CDevice, sizeof(int)*size, cudaMemcpyDeviceToHost) );

	/* Impresion final */
	printf("Array despues del kernel:");
	for(int i=0; i<10; i++){	
		printf(" %d ",CHost[i]);	
	}
	for(int i=size-10; i<size; i++){	
		printf(" %d ",CHost[i]);	
	}
	printf("\n");	

	/* Liberamos memoria del DEVICE */
	CUDA_SAFE_CALL( cudaFree(ADevice) );
	CUDA_SAFE_CALL( cudaFree(BDevice) );
	CUDA_SAFE_CALL( cudaFree(CDevice) );

	/* Liberamos los recursos asociados al device en el host */
	cudaDeviceReset();

	/* Liberamos la memoria del host */
	free(AHost);
	free(BHost);
	free(CHost);

	return 0;
}

