#!/usr/bin/python
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon - User admin
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################ 

# Sys
import sys

# Import tablon
from tablon import UserList, mkpasswd

# Import the argument parser
from argparse import ArgumentParser

# Load the current user list
userlist = UserList()





# Create the parser
parser = ArgumentParser(description='Tablon user management')
subparsers = parser.add_subparsers()

# Subparser for list
parser_list = subparsers.add_parser('list',help='Show the user list')
parser_list.set_defaults(cmd='list')

# Subparser for adduser
parser_adduser = subparsers.add_parser('adduser',help='Add a single user')
parser_adduser.set_defaults(cmd='adduser')
parser_adduser.add_argument('name', help='Name of the User')
parser_adduser.add_argument('password', nargs='?', help='The password')
parser_adduser.add_argument('--quota', help='The quota')

# Subparser for rmuser
parser_rmuser = subparsers.add_parser('rmuser',help='Remove an user')
parser_rmuser.set_defaults(cmd='rmuser')
parser_rmuser.add_argument('name', help='Name of the User')

# Subparser for addlist
parser_addlist = subparsers.add_parser('addlist',help='Add a list of users from a file')
parser_addlist.set_defaults(cmd='addlist')
parser_addlist.add_argument('list', help='List with the user naems')
parser_addlist.add_argument('--quota', nargs='?', help='The quota')

# Subparser for rmlist
parser_rmlist = subparsers.add_parser('rmlist',help='remove a list of user from a file')
parser_rmlist.set_defaults(cmd='rmlist')
parser_rmlist.add_argument('list', help='List with the user naems')

# Subparse for chpass
parser_chpassword = subparsers.add_parser('chpass',help='Change the password of an user')
parser_chpassword.set_defaults(cmd='chpass')
parser_chpassword.add_argument('name', help='Name of the User')
parser_chpassword.add_argument('password', nargs='?', help='The password')


# Subparse for chquota
parser_chquota = subparsers.add_parser('chquota',help='Change the quota of a user of an user of list of users')
parser_chquota.set_defaults(cmd='chquota')
parser_chquota.add_argument('name_or_list', help='Name of the User or list')
parser_chquota.add_argument('quota', nargs='?', help='The quota')

# Parse the arguments
args = parser.parse_args()


# Print the user list
# ----------------------------------------------- #
if args.cmd == 'list':
	print "------ USERS / Quota ------"
	for user in userlist:
		print('%s\t[%d/%d]' % (user.name,user.usedquota,user.quota))


# Create a new user
# ----------------------------------------------- #
if args.cmd == 'adduser':

	password = args.password
	if not password:
		password = mkpasswd()
	quota = args.quota
	if not quota:
		quota = 0
	else:
		quota = int(quota)
	userlist.create(args.name,password,quota)
	print "User %s, password: %s, quota %d" % (args.name, password, quota)



# Remove a user
# ----------------------------------------------- #
if args.cmd == 'rmuser':
	if userlist.remove(args.name):
		print "Removed: %s" % args.name
	else:
		print "User not found"





# Create new users from a list file 
# ----------------------------------------------- #
if args.cmd == 'addlist':

	quota = args.quota
	if not quota:
		quota = 0
	else:
		quota = int(quota)
	print "Quota: %d" % quota

	with open(args.list) as f:
		users = [line.rstrip() for line in f]
		for user in users:
			password = mkpasswd()
			userlist.create(user,password,quota)
			print user, password





# Remove users from a list file 
# ----------------------------------------------- #
if args.cmd == 'rmlist':

	with open(args.list) as f:
		notfound = []
		for line in f:
			name = line.rstrip()
			if userlist.remove(name):
				print "Removed: %s" % name
			else:
				notfound.append(name)

		if len(notfound) > 0:
			print "Not found: " + ", ".join(notfound)
	


# Change the password
# ----------------------------------------------- #
if args.cmd == 'chpass':

	name = args.name
	password = args.password
	user = userlist.get(name)
	if not user:
		print "User %s do not exists" % name
	else:
		quota = user.quota
		userlist.remove(name)
		userlist.create(name,password,quota)
		print "User %s, password: %s, quota %d" % (args.name, password, quota)



# Change the password
# ----------------------------------------------- #
if args.cmd == 'chquota':

	name_or_list = args.name_or_list
	quota = int(args.quota)


	user = userlist.get(name_or_list)
	if user:
		user.quota = quota
		userlist.save()
		print "User %s, quota %d" % (name_or_list,quota)
	else:
		with open(args.name_or_list) as f:
			notfound = []
			for line in f:
				name = line.rstrip()
				user = userlist.get(name)
				if user:
					user.quota = quota
					userlist.save()
					print "User %s, quota %d" % (name,quota)
				else:
					notfound.append(name)

			if len(notfound) > 0:
				print "Not found: " + ", ".join(notfound)





